unit LogGraph;
(*
WARNING: Don't use fractional part of GraphData[X],
   instead of that set Divisor to 1000 and all Data
   multiply by 1000 before Adding or do some similar thing.
*)

(*
 * TLogGraph v1.07 (FREEWARE component)
 * ---------------
 * Released 18 Aug 2000
 *
 * This component is suitable for showing some values, which came every
 * second, for example watching bytes received per second, CPU usage etc.
 *
 * You can use it freely in your programs, but if you do so please write my
 * name in About box or somewhere alike.
 * If you think that this compoent is useful, you can send me an e-mail.
 * Welcomed are also another messages like bug reports.
 *
 * What's new
 * ----------
 * 1.07 - added support for level line (modified by Aleq)
 * 1.06 - changed data type from Integer to Comp
 * 1.05 - corrected procedure SetBevel
 *      - corrected procedure DrawToScreen
 *      - this unit no longer requires TileImage.pas, when you don't want to
 *        work with tiled images.
 *        If you want to use TTileImage with this component, you need some newer
 *        version of TTileImage than 1.0 (it should be also faster)
 *      - Bevel is now of type TPanelBevel
 * 1.01 - XTempBitmap is created only in procedure DrawToScreen
 * 1.00 - initial release
 *
 *
 * Properties:
 *   AfterString     Text displayed behind a number.
 *   BetweenSpace    Number of pixels between two values plus one
 *   ColorBg         Background color
 *   ColorFg         Foreground (graph) color
 *   ColorText       Text color
 *   DescString      This text is displayed only when ViewStyle=vsLabels
 *   DispUpper       True if text at the top is visible (try it and you will see)
 *   DispAfter       True if AfterString is visible
 *   Divisor         This compoent can handle only Integer-type values, but you can use
 *                   Divisor to display this values in another unit
 *                   Example: set Divisor to 1024 to display bytes in kB
 *   Image           Background Image
 *   TileImage       Another background image
 *   ValueMax        Max. value visible in the graph
 *   ValueMin        Min. value visible in the graph
 *   ViewStyle       You MUST try it
 *
 *   GraphData       values in graph in array [0..ValuesCount-1]
 *      
 *)

interface

{$DEFINE Use_TTileImage}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls
{$IFDEF Use_TTileImage}
  ,TileImage
{$ENDIF}
  ;

const
  MaxDataCount=1000;
  StdDataCount=30;

type
  TGraphData=Array [0..MaxDataCount-1] of Comp;
  TViewStyle=(vsGraph,vsLabels);
{  TBevel=(bvNone,bvRaised,bvLowered);}

type
  TLogGraph = class(TGraphicControl)
  private
    FGraphData:TGraphData;
    FIntDataMax:Integer;
    FIntDataMin:Integer;
    FColorBg:TColor;
    FColorFg:TColor;
    FColorText:TColor;
    FBetweenSpace:Byte;
    FAfterString:ShortString;
    FDivisor:Integer;
    FViewStyle:TViewStyle;
    FDescString:String;
    FBevel:TPanelBevel;
    FDispUpper:Boolean;
    FDispAfter:Boolean;
    FImage:TImage;
    FLinePos:Integer;
    FLinePen:TPen;
    FShowLine:Boolean;
{$IFDEF Use_TTileImage}
    FTileImage:TTileImage;
{$ENDIF}
    XOneStep:Real;
    XRealZero:Real;
    XIntZero:Integer;
    XDestR,XSourceR,XAllR:TRect;
    XCount:Integer;
    XLastWidth,XLastHeight:Integer;
    function GetGraphData (I:Byte):Integer;
    procedure SetGraphData (I:Byte;Value:Integer);
    procedure SetDataMax (Value:Integer);
    procedure SetDataMin (Value:Integer);
    procedure SetColorBg(Value: TColor);
    procedure SetColorFg(Value: TColor);
    procedure SetBetweenSpace (Value:Byte);
    procedure SetAfterString (Value:ShortString);
    procedure SetDivisor (Value:Integer);
    procedure SetColorText (Value:TColor);
    procedure SetViewStyle (Value:TViewStyle);
    procedure SetDescString (Value:String);
    procedure SetDispUpper (Value:Boolean);
    procedure SetDispAfter (Value:Boolean);
    procedure SetImage (Value:TImage);
{$IFDEF Use_TTileImage}
    procedure SetTileImage (Value:TTileImage);
{$ENDIF}
    procedure SetBevel (Value:TPanelBevel);
    procedure SetLinePos(Value: Integer);
    procedure SetLinePen(Value: TPen);
    procedure SetShowLine(Value: Boolean);
  protected
    procedure InitXValues;
    procedure DrawToScreen;
    procedure Paint; override;
  public
    property GraphData[I:Byte]:Integer read GetGraphData write SetGraphData;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Add (Value:Comp);
  published
    property AfterString:ShortString read FAfterString write SetAfterString;
    property BetweenSpace:Byte read FBetweenSpace write SetBetweenSpace;
    property Bevel:TPanelBevel read FBevel write SetBevel;
    property ColorBg:TColor read FColorBg write SetColorBg;
    property ColorFg:TColor read FColorFg write SetColorFg;
    property ColorText:TColor read FColorText write SetColorText;
    property DescString:String read FDescString write SetDescString;
    property DispUpper:Boolean read FDispUpper write SetDispUpper;
    property DispAfter:Boolean read FDispAfter write SetDispAfter;
    property Divisor:Integer read FDivisor write SetDivisor;
    property Image:TImage read FImage write SetImage;
    property LinePen: TPen read FLinePen write SetLinePen;
    property LinePos: Integer read FLinePos write SetLinePos;    
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property ShowLine: Boolean read FShowLine write SetShowLine;
{$IFDEF Use_TTileImage}
    property TileImage:TTileImage read FTileImage write SetTileImage;
{$ENDIF}
    property ValuesCount:Integer read XCount;
    property ValueMax:Integer read FIntDataMax write SetDataMax;
    property ValueMin:Integer read FIntDataMin write SetDataMin;
    property ViewStyle:TViewStyle read FViewStyle write SetViewStyle;
    property Visible;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

{{$R LogGraphRes.dcr}

procedure TLogGraph.SetColorText (Value:TColor);
begin
  FColorText:=Value;
  DrawToScreen;
end;

procedure TLogGraph.SetViewStyle (Value:TViewStyle);
begin
  If Value<>FViewStyle then
  begin
    FViewStyle:=Value;
    Invalidate;
  end;
end;

procedure TLogGraph.SetDescString (Value:String);
begin
  FDescString:=Value;
  If ViewStyle=vsLabels then
    Invalidate;
end;

procedure TLogGraph.DrawToScreen;
var S:String;
    A,C,Ind:Integer;
    B:Comp;
    XTempBitmap:TBitmap;

  procedure DrawUp;
  begin
    with XTempBitmap.Canvas do
    begin
      MoveTo (0,Height-1);
      LineTo (0,0);
      LineTo (Width-1,0);
    end;
  end;

  procedure DrawDown;
  begin
    with XTempBitmap.Canvas do
    begin
      MoveTo (1,Height-1);
      LineTo (Width-1,Height-1);
      LineTo (Width-1,0);
    end;
  end;

begin
  If (XLastHeight<>Height) OR (XLastWidth<>Width) then
    InitXValues;
  XLastHeight:=Height;
  XLastWidth:=Width;
  XTempBitmap:=TBitmap.Create;
  try
    XTempBitmap.Width:=Width;
    XTempBitmap.Height:=Height;
    XTempBitmap.Canvas.Font.Color:=FColorText;
    XTempBitmap.Canvas.Brush.Color:=FColorBg;
{$IFDEF Use_TTileImage}
    If (FTileImage<>nil) AND (FTileImage.Picture.Graphic<>nil) then
      FTileImage.DrawToCanvas (XTempBitmap.Canvas,Left,Top,Width,Height)
    else
{$ENDIF}
      If (FImage<>nil) AND (FImage.Picture.Graphic<>nil) then
        XTempBitmap.Canvas.Draw (FImage.Left-Left,FImage.Top-Top,FImage.Picture.Graphic)
      else
      begin
        XTempBitmap.Canvas.Brush.Color:=FColorBg;
        XTempBitmap.Canvas.FillRect (XAllR);
      end;
    XTempBitmap.Canvas.Brush.Style:=bsClear;
    If FBevel<>bvNone then
    begin
      with XTempBitmap.Canvas do
      begin
        Pen.Style:=psSolid;
        Pen.Color:=clBtnHighlight;
        If FBevel=bvRaised then
          DrawUp
        else
          DrawDown;
        Pen.Color:=clBtnShadow;
        If FBevel=bvRaised then
          DrawDown
        else
          DrawUp;
      end;
    end;
    Case ViewStyle of
      vsGraph:
        begin
          XTempBitmap.Canvas.Pen.Color:=FColorFg;
          XTempBitmap.Canvas.Pen.Style:=psSolid;
          If FBevel=bvNone then
            A:=0
          else
            A:=1;
          For Ind:=0 to XCount-1 do
          begin
            If FGraphData[Ind]=0 then
              B:=XIntZero
            else
            begin
              If FGraphData[Ind]>FIntDataMax then
                B:=FIntDataMax
              else
                If FGraphData[Ind]<FIntDataMin then
                  B:=FIntDataMin
                else
                  B:=FGraphData[Ind];
              If FBevel=bvNone then
                B:=Round((FIntDataMax-B)*XOneStep)-1
              else
                B:=Round((FIntDataMax-B)*XOneStep);
            end;
            If B<>XIntZero then
            begin
              XTempBitmap.Canvas.MoveTo (A,XIntZero);
              XTempBitmap.Canvas.LineTo (A,Round(B));
            end
            else
              XTempBitmap.Canvas.Pixels[A,XIntZero]:=FColorFg;
            Inc (A,FBetweenSpace);
          end;

          if FShowLine then
          begin
            XTempBitmap.Canvas.Pen.Assign(FLinePen);
            If FBevel=bvNone then A:=Round((FIntDataMax-FLinePos)*XOneStep)-1 else
                                  A:=Round((FIntDataMax-FLinePos)*XOneStep);
            XTempBitmap.Canvas.MoveTo(0, A);
            XTempBitmap.Canvas.LineTo(Width, A);
          end;

          If (FDivisor<>0) AND (FDispUpper) then
          begin
            If FDivisor<>1 then
            begin
              S:=IntToStr(Round(FGraphData[XCount-1]/FDivisor*10));
              Insert ('.',S,Length(S));
              If ((S[1]='-') AND (Length(S)=3)) OR
                 (Length(S)=2) then
                Insert ('0',S,Length(S)-1);
            end
            else
              S:=IntToStr (Round(FGraphData[XCount-1]));
            If FDispAfter then
            begin
              If (AfterString<>'') AND (AfterString[1]<>' ') then
                S:=S+' ';
              S:=S+FAfterString;
            end;
            If FBevel=bvNone then
              A:=0
            else
              A:=1;
            Canvas.Draw(0,0,XTempBitmap);
            Canvas.Font.Color:=FColorText;
            Canvas.Brush.Style:=bsClear;
            Canvas.TextOut (Width div 2-Canvas.TextWidth(S) div 2,A,S);
          end
          else
            Canvas.Draw(0,0,XTempBitmap);
        end;
      vsLabels:
        begin
          A:=XTempBitmap.Canvas.TextHeight ('Xy');
          If FBevel=bvNone then
            C:=0
          else
            C:=1;
          If FDispUpper then
          begin
            If XTempBitmap.Canvas.TextWidth (FDescString)>XAllR.Right-XAllR.Left-1 then
            begin
              S:=FDescString+'.';
              repeat
                Delete (S,Length(S)-1,2);
                S:=S+'.';
              until (Length(S)=1) OR (XTempBitmap.Canvas.TextWidth (S)<=XAllR.Right-XAllR.Left-1);
              while (Length(S)>1) AND (S[Length(S)-1]=' ') do
                Delete (S,Length(S)-1,1);
            end
            else
              S:=FDescString;
            XTempBitmap.Canvas.TextOut (Width div 2-XTempBitmap.Canvas.TextWidth(S) div 2,C,S);
          end;
          If FDivisor<>0 then
          begin
            S:=IntToStr(Round(FGraphData[XCount-1]/FDivisor*10));
            Insert ('.',S,Length(S));
            If ((S[1]='-') AND (Length(S)=3)) OR
               (Length(S)=2) then
              Insert ('0',S,Length(S)-1);
            XTempBitmap.Canvas.TextOut (Width div 2-XTempBitmap.Canvas.TextWidth(S) div 2,Height div 2-A div 2,S);
          end;
          If FDispAfter then
          begin
            If FBevel=bvNone then
              C:=0
            else
              C:=1;
            XTempBitmap.Canvas.TextOut (Width div 2-XTempBitmap.Canvas.TextWidth(FAfterString) div 2,Height-A-C,FAfterString);
          end;
          Canvas.Draw(0,0,XTempBitmap);
        end;
    end; {Case}
  finally
    XTempBitmap.Free;
  end;
end;

procedure TLogGraph.SetDispUpper (Value:Boolean);
begin
  If Value<>FDispUpper then
  begin
    FDispUpper:=Value;
    Invalidate;
  end;
end;

procedure TLogGraph.SetDispAfter (Value:Boolean);
begin
  If Value<>FDispAfter then
  begin
    FDispAfter:=Value;
    Invalidate;
  end;
end;

procedure TLogGraph.SetDivisor (Value:Integer);
begin
  If Value<>FDivisor then
  begin
    FDivisor:=Value;
    Invalidate;
  end;
end;

procedure TLogGraph.SetAfterString (Value:ShortString);
begin
  If Value<>FAfterString then
  begin
    FAfterString:=Value;
    Invalidate;
  end;
end;

procedure TLogGraph.SetBetweenSpace (Value:Byte);
begin
  If Value>0 then
  begin
    If Value<>FBetweenSpace then
    begin
      FBetweenSpace:=Value;
      InitXValues;
      DrawToScreen;
    end;
  end
  else
    If csDesigning in ComponentState then
      MessageDlg ('Value must be greater than 0 !',mtError,[mbOK],0);
end;

procedure TLogGraph.SetColorBg(Value: TColor);
begin
  If FColorBg<>Value then
  begin
    FColorBg:=Value;
    Invalidate;
  end;
end;

procedure TLogGraph.SetColorFg(Value: TColor);
begin
  If FColorFg<>Value then
  begin
    FColorFg:=Value;
    Invalidate;
  end;
end;

procedure TLogGraph.InitXValues;
begin
  If Width div FBetweenSpace>MaxDataCount then
    Width:=MaxDataCount div FBetweenSpace;
  If FBevel=bvNone then
  begin
    XOneStep:=Height/(FIntDataMax-FIntDataMin+1);
    XRealZero:=XOneStep*(FIntDataMax+1)-1;
    XIntZero:=Round(XRealZero);
    XSourceR:=Rect (FBetweenSpace,0,Width,Height);
    XDestR:=Rect (0,0,Width-FBetweenSpace,Height);
    XAllR:=Rect (0,0,Width,Height);
    XCount:=(Width div FBetweenSpace);
  end
  else
  begin
    XOneStep:=(Height-2)/(FIntDataMax-FIntDataMin+1);
    XRealZero:=XOneStep*(FIntDataMax+1)-1+1;
    XIntZero:=Round(XRealZero);
    XSourceR:=Rect (FBetweenSpace+1,1,Width-1,Height-1);
    XDestR:=Rect (1,1,Width-FBetweenSpace-1,Height-1);
    XAllR:=Rect (1,1,Width-1,Height-1);
    XCount:=(Width-2) div FBetweenSpace;
  end;
end;

procedure TLogGraph.Add (Value:Comp);
begin
  System.Move (FGraphData[1],FGraphData[0],(XCount-1) * SizeOf(Value));
  FGraphData[XCount-1]:=Value;
  If Visible then
    DrawToScreen;
end;

procedure TLogGraph.SetDataMax (Value:Integer);
begin
  If Value<>FIntDataMax then
  begin
    FIntDataMax:=Value;
    InitXValues;
    Invalidate;
  end;
end;

procedure TLogGraph.SetDataMin (Value:Integer);
begin
  If Value<>FIntDataMin then
  begin
    FIntDataMin:=Value;
    InitXValues;
    Invalidate;
  end;
end;

function TLogGraph.GetGraphData (I:Byte):Integer;
begin
  If (I<XCount) then
    Result:=Round(FGraphData[I])
  else
    Result:=0;  
end;

procedure TLogGraph.SetGraphData (I:Byte;Value:Integer);
begin
  If Value<>FGraphData[I] then
  begin
    FGraphData[I]:=Value;
    DrawToScreen;
  end;
end;

procedure TLogGraph.Paint;
begin
  DrawToScreen;
end;

constructor TLogGraph.Create(AOwner: TComponent);
var A:Integer;
begin
  inherited Create (AOwner);
  For A:=0 to MaxDataCount-1 do
    FGraphData[A]:=0;
  FImage:=nil;
{$IFDEF Use_TTileImage}
  FTileImage:=nil;
{$ENDIF}
  FDispUpper:=True;
  FDispAfter:=True;
  FViewStyle:=vsGraph;
  FIntDataMax:=100;
  FIntDataMin:=-20;
  FBetweenSpace:=3;
  FDivisor:=1;
  FBevel:=bvRaised;
  Width:=(StdDataCount-1)*FBetweenSpace+1;
  Height:=65;
  XLastHeight:=0;
  XLastWidth:=0;
  FColorBg:=clBtnFace;
  FColorFg:=clRed;
  FColorText:=clWindowText;
  Canvas.Brush.Color:=FColorBg;
  FLinePen:=TPen.Create;
  FLinePos:=0;
  FShowLine:=False;
  InitXValues;
end;

destructor TLogGraph.Destroy;
begin
  FLinePen.Free;
  inherited Destroy;
end;

procedure TLogGraph.SetImage (Value:TImage);
begin
  If FImage<>Value then
  begin
    FImage:=Value;
    Invalidate;
  end;
end;

{$IFDEF Use_TTileImage}
procedure TLogGraph.SetTileImage (Value:TTileImage);
begin
  If FTileImage<>Value then
  begin
    FTileImage:=Value;
    Invalidate;
  end;
end;
{$ENDIF}

procedure TLogGraph.SetBevel (Value:TPanelBevel);
begin
  If Value<>FBevel then
  begin
    FBevel:=Value;
    InitXValues;
    Invalidate;
  end;
end;

procedure TLogGraph.SetLinePos(Value: Integer);
begin
  FLinePos:=Value;
  DrawToScreen;
end;

procedure TLogGraph.SetLinePen(Value: TPen);
begin
  FLinePen.Assign(Value);
  DrawToScreen;
end;

procedure TLogGraph.SetShowLine(Value: Boolean);
begin
  FShowLine:=Value;
  DrawToScreen;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TLogGraph]);
end;

end.
